/*
Copyright (C) 1994-1995 Apogee Software, Ltd.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#ifndef _sprites_public
#define _sprites_public

#include "develop.h"

typedef enum {ACTOR_START_LABEL,


/***************    UNTERWACHE  *************************************/


SPR_LOWGRD_SHOOT1,SPR_LOWGRD_SHOOT2,SPR_LOWGRD_SHOOT3,SPR_LOWGRD_SHOOT4,

SPR_LOWGRD_S1,SPR_LOWGRD_S2,SPR_LOWGRD_S3,SPR_LOWGRD_S4,
SPR_LOWGRD_S5,SPR_LOWGRD_S6,SPR_LOWGRD_S7,SPR_LOWGRD_S8,

SPR_LOWGRD_W11,SPR_LOWGRD_W12,SPR_LOWGRD_W13,SPR_LOWGRD_W14,
SPR_LOWGRD_W15,SPR_LOWGRD_W16,SPR_LOWGRD_W17,SPR_LOWGRD_W18,

SPR_LOWGRD_W21,SPR_LOWGRD_W22,SPR_LOWGRD_W23,SPR_LOWGRD_W24,
SPR_LOWGRD_W25,SPR_LOWGRD_W26,SPR_LOWGRD_W27,SPR_LOWGRD_W28,

SPR_LOWGRD_W31,SPR_LOWGRD_W32,SPR_LOWGRD_W33,SPR_LOWGRD_W34,
SPR_LOWGRD_W35,SPR_LOWGRD_W36,SPR_LOWGRD_W37,SPR_LOWGRD_W38,

SPR_LOWGRD_W41,SPR_LOWGRD_W42,SPR_LOWGRD_W43,SPR_LOWGRD_W44,
SPR_LOWGRD_W45,SPR_LOWGRD_W46,SPR_LOWGRD_W47,SPR_LOWGRD_W48,


SPR_LOWGRD_PAIN1, SPR_LOWGRD_PAIN2, SPR_LOWGRD_DIE1, SPR_LOWGRD_DIE2,
SPR_LOWGRD_DIE3, SPR_LOWGRD_DIE4, SPR_LOWGRD_DEAD,

SPR_LOWGRD_WPAIN1, SPR_LOWGRD_WPAIN2, SPR_LOWGRD_WDIE1, SPR_LOWGRD_WDIE2,
SPR_LOWGRD_WDIE3, SPR_LOWGRD_WDIE4, SPR_LOWGRD_WDEAD,


SPR_SNEAKY_DEAD, SPR_RISE1, SPR_RISE2, SPR_RISE3, SPR_RISE4,

/*
SPR_LOWGRD_USE11,SPR_LOWGRD_USE12,SPR_LOWGRD_USE13,SPR_LOWGRD_USE14,
SPR_LOWGRD_USE15,SPR_LOWGRD_USE16,SPR_LOWGRD_USE17,SPR_LOWGRD_USE18,

SPR_LOWGRD_USE21,SPR_LOWGRD_USE22,SPR_LOWGRD_USE23,SPR_LOWGRD_USE24,
SPR_LOWGRD_USE25,SPR_LOWGRD_USE26,SPR_LOWGRD_USE27,SPR_LOWGRD_USE28,
*/


/*****************  Highguard 1 (Steve B.)  ***************************************/




SPR_HIGHGRD_SHOOT1,SPR_HIGHGRD_SHOOT2,SPR_HIGHGRD_SHOOT3,SPR_HIGHGRD_SHOOT4,

SPR_HIGHGRD_S1,SPR_HIGHGRD_S2,SPR_HIGHGRD_S3,SPR_HIGHGRD_S4,
SPR_HIGHGRD_S5,SPR_HIGHGRD_S6,SPR_HIGHGRD_S7,SPR_HIGHGRD_S8,

SPR_HIGHGRD_W11,SPR_HIGHGRD_W12,SPR_HIGHGRD_W13,SPR_HIGHGRD_W14,
SPR_HIGHGRD_W15,SPR_HIGHGRD_W16,SPR_HIGHGRD_W17,SPR_HIGHGRD_W18,

SPR_HIGHGRD_W21,SPR_HIGHGRD_W22,SPR_HIGHGRD_W23,SPR_HIGHGRD_W24,
SPR_HIGHGRD_W25,SPR_HIGHGRD_W26,SPR_HIGHGRD_W27,SPR_HIGHGRD_W28,

SPR_HIGHGRD_W31,SPR_HIGHGRD_W32,SPR_HIGHGRD_W33,SPR_HIGHGRD_W34,
SPR_HIGHGRD_W35,SPR_HIGHGRD_W36,SPR_HIGHGRD_W37,SPR_HIGHGRD_W38,

SPR_HIGHGRD_W41,SPR_HIGHGRD_W42,SPR_HIGHGRD_W43,SPR_HIGHGRD_W44,
SPR_HIGHGRD_W45,SPR_HIGHGRD_W46,SPR_HIGHGRD_W47,SPR_HIGHGRD_W48,


SPR_HIGHGRD_PAIN1,SPR_HIGHGRD_PAIN2,
SPR_HIGHGRD_DIE1,SPR_HIGHGRD_DIE2,SPR_HIGHGRD_DIE3,SPR_HIGHGRD_DIE4,
SPR_HIGHGRD_DIE5,SPR_HIGHGRD_DEAD,

SPR_HIGHGRD_WPAIN1,SPR_HIGHGRD_WPAIN2,
SPR_HIGHGRD_WDIE1,SPR_HIGHGRD_WDIE2,SPR_HIGHGRD_WDIE3,SPR_HIGHGRD_WDIE4,
SPR_HIGHGRD_WDIE5,SPR_HIGHGRD_WDEAD,

/*
SPR_HIGHGRD_USE11,SPR_HIGHGRD_USE12,SPR_HIGHGRD_USE13,SPR_HIGHGRD_USE14,
SPR_HIGHGRD_USE15,SPR_HIGHGRD_USE16,SPR_HIGHGRD_USE17,SPR_HIGHGRD_USE18,

SPR_HIGHGRD_USE21,SPR_HIGHGRD_USE22,SPR_HIGHGRD_USE23,SPR_HIGHGRD_USE24,
SPR_HIGHGRD_USE25,SPR_HIGHGRD_USE26,SPR_HIGHGRD_USE27,SPR_HIGHGRD_USE28,
*/




/******************** ANGRIFFSTUPPE **********************************/


SPR_STRIKE_SHOOT1,SPR_STRIKE_SHOOT2,SPR_STRIKE_SHOOT3,SPR_STRIKE_SHOOT4,
SPR_STRIKE_KSHOOT1,SPR_STRIKE_KSHOOT2,SPR_STRIKE_KSHOOT3,


SPR_STRIKE_S1,SPR_STRIKE_S2,SPR_STRIKE_S3,SPR_STRIKE_S4,
SPR_STRIKE_S5,SPR_STRIKE_S6,SPR_STRIKE_S7,SPR_STRIKE_S8,

SPR_STRIKE_W11,SPR_STRIKE_W12,SPR_STRIKE_W13,SPR_STRIKE_W14,
SPR_STRIKE_W15,SPR_STRIKE_W16,SPR_STRIKE_W17,SPR_STRIKE_W18,

SPR_STRIKE_W21,SPR_STRIKE_W22,SPR_STRIKE_W23,SPR_STRIKE_W24,
SPR_STRIKE_W25,SPR_STRIKE_W26,SPR_STRIKE_W27,SPR_STRIKE_W28,

SPR_STRIKE_W31,SPR_STRIKE_W32,SPR_STRIKE_W33,SPR_STRIKE_W34,
SPR_STRIKE_W35,SPR_STRIKE_W36,SPR_STRIKE_W37,SPR_STRIKE_W38,

SPR_STRIKE_W41,SPR_STRIKE_W42,SPR_STRIKE_W43,SPR_STRIKE_W44,
SPR_STRIKE_W45,SPR_STRIKE_W46,SPR_STRIKE_W47,SPR_STRIKE_W48,


SPR_STRIKE_PAIN1,SPR_STRIKE_PAIN2,
SPR_STRIKE_DIE1,SPR_STRIKE_DIE2,SPR_STRIKE_DIE3,
SPR_STRIKE_DIE4,SPR_STRIKE_DEAD1,SPR_STRIKE_DEAD2,SPR_STRIKE_DEAD3,

SPR_STRIKE_WPAIN1,SPR_STRIKE_WPAIN2,
SPR_STRIKE_WDIE1,SPR_STRIKE_WDIE2,SPR_STRIKE_WDIE3,
SPR_STRIKE_WDIE4,SPR_STRIKE_WDEAD1,SPR_STRIKE_WDEAD2,SPR_STRIKE_WDEAD3,

SPR_STRIKE_RROLL1,SPR_STRIKE_RROLL2,SPR_STRIKE_RROLL3,SPR_STRIKE_RROLL4,
SPR_STRIKE_RROLL5,SPR_STRIKE_RROLL6,

SPR_STRIKE_LROLL1,SPR_STRIKE_LROLL2,SPR_STRIKE_LROLL3,SPR_STRIKE_LROLL4,
SPR_STRIKE_LROLL5,SPR_STRIKE_LROLL6,




/********************** BLITZWACHE **********************************/


SPR_BLITZ_SHOOT1,SPR_BLITZ_SHOOT2,SPR_BLITZ_SHOOT3,SPR_BLITZ_SHOOT4,

SPR_BLITZ_S1,SPR_BLITZ_S2,SPR_BLITZ_S3,SPR_BLITZ_S4,
SPR_BLITZ_S5,SPR_BLITZ_S6,SPR_BLITZ_S7,SPR_BLITZ_S8,

SPR_BLITZ_W11,SPR_BLITZ_W12,SPR_BLITZ_W13,SPR_BLITZ_W14,
SPR_BLITZ_W15,SPR_BLITZ_W16,SPR_BLITZ_W17,SPR_BLITZ_W18,

SPR_BLITZ_W21,SPR_BLITZ_W22,SPR_BLITZ_W23,SPR_BLITZ_W24,
SPR_BLITZ_W25,SPR_BLITZ_W26,SPR_BLITZ_W27,SPR_BLITZ_W28,

SPR_BLITZ_W31,SPR_BLITZ_W32,SPR_BLITZ_W33,SPR_BLITZ_W34,
SPR_BLITZ_W35,SPR_BLITZ_W36,SPR_BLITZ_W37,SPR_BLITZ_W38,

SPR_BLITZ_W41,SPR_BLITZ_W42,SPR_BLITZ_W43,SPR_BLITZ_W44,
SPR_BLITZ_W45,SPR_BLITZ_W46,SPR_BLITZ_W47,SPR_BLITZ_W48,


SPR_BLITZ_PAIN1, SPR_BLITZ_PAIN2,SPR_BLITZ_DIE1,SPR_BLITZ_DIE2,
SPR_BLITZ_DIE3,SPR_BLITZ_DIE4,SPR_BLITZ_DEAD1, SPR_BLITZ_DEAD2,

SPR_BLITZ_WPAIN1, SPR_BLITZ_WPAIN2,SPR_BLITZ_WDIE1,SPR_BLITZ_WDIE2,
SPR_BLITZ_WDIE3,SPR_BLITZ_WDIE4,SPR_BLITZ_WDEAD1, SPR_BLITZ_WDEAD2,

SPR_BLITZ_RISE1, SPR_BLITZ_RISE2, SPR_BLITZ_RISE3, SPR_BLITZ_RISE4,

SPR_BLITZ_USE11,SPR_BLITZ_USE12,SPR_BLITZ_USE13,SPR_BLITZ_USE14,
SPR_BLITZ_USE15,SPR_BLITZ_USE16,SPR_BLITZ_USE17,SPR_BLITZ_USE18,

SPR_BLITZ_USE21,SPR_BLITZ_USE22,SPR_BLITZ_USE23,SPR_BLITZ_USE24,
SPR_BLITZ_USE25,SPR_BLITZ_USE26,SPR_BLITZ_USE27,SPR_BLITZ_USE28,

SPR_BLITZ_PLEAD1,SPR_BLITZ_PLEAD2,SPR_BLITZ_PLEAD3,SPR_BLITZ_PLEAD4,
SPR_BLITZ_PLEAD5,SPR_BLITZ_PLEAD6,SPR_BLITZ_PLEAD7,SPR_BLITZ_PLEAD8,
SPR_BLITZ_PLEAD9,SPR_BLITZ_PLEAD10,SPR_BLITZ_PLEAD11,




/******************** DREIHEITWACHE ***********************************/


SPR_ENFORCER_SHOOT1,SPR_ENFORCER_SHOOT2,SPR_ENFORCER_SHOOT3,
SPR_ENFORCER_SHOOT4,

SPR_ENFORCER_S1,SPR_ENFORCER_S2,SPR_ENFORCER_S3,SPR_ENFORCER_S4,
SPR_ENFORCER_S5,SPR_ENFORCER_S6,SPR_ENFORCER_S7,SPR_ENFORCER_S8,

SPR_ENFORCER_W11,SPR_ENFORCER_W12,SPR_ENFORCER_W13,SPR_ENFORCER_W14,
SPR_ENFORCER_W15,SPR_ENFORCER_W16,SPR_ENFORCER_W17,SPR_ENFORCER_W18,

SPR_ENFORCER_W21,SPR_ENFORCER_W22,SPR_ENFORCER_W23,SPR_ENFORCER_W24,
SPR_ENFORCER_W25,SPR_ENFORCER_W26,SPR_ENFORCER_W27,SPR_ENFORCER_W28,

SPR_ENFORCER_W31,SPR_ENFORCER_W32,SPR_ENFORCER_W33,SPR_ENFORCER_W34,
SPR_ENFORCER_W35,SPR_ENFORCER_W36,SPR_ENFORCER_W37,SPR_ENFORCER_W38,

SPR_ENFORCER_W41,SPR_ENFORCER_W42,SPR_ENFORCER_W43,SPR_ENFORCER_W44,
SPR_ENFORCER_W45,SPR_ENFORCER_W46,SPR_ENFORCER_W47,SPR_ENFORCER_W48,


SPR_ENFORCER_THROW1,SPR_ENFORCER_THROW2,SPR_ENFORCER_THROW3,SPR_ENFORCER_THROW4,
SPR_ENFORCER_THROW5,SPR_ENFORCER_THROW6,SPR_ENFORCER_THROW7,SPR_ENFORCER_THROW8,

SPR_ENFORCER_PAIN1,SPR_ENFORCER_PAIN2,
SPR_ENFORCER_DIE1,SPR_ENFORCER_DIE2,SPR_ENFORCER_DIE3,SPR_ENFORCER_DIE4,
SPR_ENFORCER_DEAD,

SPR_ENFORCER_WPAIN1,SPR_ENFORCER_WPAIN2,
SPR_ENFORCER_WDIE1,SPR_ENFORCER_WDIE2,SPR_ENFORCER_WDIE3,SPR_ENFORCER_WDIE4,
SPR_ENFORCER_WDEAD,

/*
SPR_ENFORCER_USE11,SPR_ENFORCER_USE12,SPR_ENFORCER_USE13,SPR_ENFORCER_USE14,
SPR_ENFORCER_USE15,SPR_ENFORCER_USE16,SPR_ENFORCER_USE17,SPR_ENFORCER_USE18,

SPR_ENFORCER_USE21,SPR_ENFORCER_USE22,SPR_ENFORCER_USE23,SPR_ENFORCER_USE24,
SPR_ENFORCER_USE25,SPR_ENFORCER_USE26,SPR_ENFORCER_USE27,SPR_ENFORCER_USE28,
*/


/************************* ROBOT GUARD *********************************/


SPR_ROBOGRD_S11,SPR_ROBOGRD_S12,SPR_ROBOGRD_S13,SPR_ROBOGRD_S14,
SPR_ROBOGRD_S15,SPR_ROBOGRD_S16,SPR_ROBOGRD_S17,SPR_ROBOGRD_S18,

SPR_ROBOGRD_S21,SPR_ROBOGRD_S22,SPR_ROBOGRD_S23,SPR_ROBOGRD_S24,
SPR_ROBOGRD_S25,SPR_ROBOGRD_S26,SPR_ROBOGRD_S27,SPR_ROBOGRD_S28,

ROBOGRDDIE1,ROBOGRDDIE2,ROBOGRDDIE3,ROBOGRDDIE4,
ROBOGRDDIE5,ROBOGRDDIE6,ROBOGRDDIE7,ROBOGRDDIE8,
ROBOGRDDIE9,ROBOGRDDEAD,



/************************* EXPLOSIONS *******************************/

SPR_EXPLOSION1,SPR_EXPLOSION2,SPR_EXPLOSION3,SPR_EXPLOSION4,
SPR_EXPLOSION5,SPR_EXPLOSION6,SPR_EXPLOSION7,SPR_EXPLOSION8,
SPR_EXPLOSION9,SPR_EXPLOSION10,SPR_EXPLOSION11,SPR_EXPLOSION12,
SPR_EXPLOSION13,SPR_EXPLOSION14,SPR_EXPLOSION15,SPR_EXPLOSION16,
SPR_EXPLOSION17,SPR_EXPLOSION18,SPR_EXPLOSION19,SPR_EXPLOSION20,



SPR_STATICEXPL1,SPR_STATICEXPL2,SPR_STATICEXPL3,SPR_STATICEXPL4,
SPR_STATICEXPL5,SPR_STATICEXPL6,SPR_STATICEXPL7,SPR_STATICEXPL8,
SPR_STATICEXPL9,SPR_STATICEXPL10,SPR_STATICEXPL11,SPR_STATICEXPL12,
SPR_STATICEXPL13,SPR_STATICEXPL14,SPR_STATICEXPL15,SPR_STATICEXPL16,
SPR_STATICEXPL17,SPR_STATICEXPL18,SPR_STATICEXPL19,SPR_STATICEXPL20,
SPR_STATICEXPL21,SPR_STATICEXPL22,SPR_STATICEXPL23,SPR_STATICEXPL24,
SPR_STATICEXPL25,

SPR_GROUNDEXPL1,SPR_GROUNDEXPL2,SPR_GROUNDEXPL3,SPR_GROUNDEXPL4,
SPR_GROUNDEXPL5,SPR_GROUNDEXPL6,SPR_GROUNDEXPL7,SPR_GROUNDEXPL8,
SPR_GROUNDEXPL9,SPR_GROUNDEXPL10,SPR_GROUNDEXPL11,SPR_GROUNDEXPL12,
SPR_GROUNDEXPL13,SPR_GROUNDEXPL14,SPR_GROUNDEXPL15,SPR_GROUNDEXPL16,
SPR_GROUNDEXPL17,SPR_GROUNDEXPL18,SPR_GROUNDEXPL19,SPR_GROUNDEXPL20,


/************************* HAZARDS *******************************/



UBLADE1,UBLADE2,UBLADE3,UBLADE4,
UBLADE5,UBLADE6,UBLADE7,UBLADE8,
UBLADE9,//UBLADE10,UBLADE11,UBLADE12,
//UBLADE13,UBLADE14,UBLADE15,UBLADE16,




FIREJETUP1,FIREJETUP2,FIREJETUP3,FIREJETUP4,
FIREJETUP5,FIREJETUP6,FIREJETUP7,FIREJETUP8,
FIREJETUP9,FIREJETUP10,FIREJETUP11,FIREJETUP12,
FIREJETUP13,FIREJETUP14,FIREJETUP15,FIREJETUP16,
FIREJETUP17,FIREJETUP18,FIREJETUP19,FIREJETUP20,
FIREJETUP21,FIREJETUP22,FIREJETUP23,



CRUSHDOWN1,CRUSHDOWN2,CRUSHDOWN3,CRUSHDOWN4,
CRUSHDOWN5,CRUSHDOWN6,CRUSHDOWN7,CRUSHDOWN8,
//CRUSHDOWN9,CRUSHDOWN10,CRUSHDOWN11,CRUSHDOWN12,
//CRUSHDOWN13,CRUSHDOWN14,CRUSHDOWN15,CRUSHDOWN16,



SPEARUP1,SPEARUP2,SPEARUP3,SPEARUP4,
SPEARUP5,SPEARUP6,SPEARUP7,SPEARUP8,
SPEARUP9,SPEARUP10,SPEARUP11,SPEARUP12,
SPEARUP13,SPEARUP14,SPEARUP15,SPEARUP16,





SPRING1,SPRING2,SPRING3,SPRING4,
SPRING5,SPRING6,SPRING7,SPRING8,
SPRING9,SPRING10,




FIREW1,FIREW2,FIREW3,FIREW4,FIREW5,FIREW6,FIREW7,FIREW8,
FIREW9,FIREW10,FIREW11,FIREW12,FIREW13,FIREW14,FIREW15,




GUTS1,GUTS2,GUTS3,GUTS4,
GUTS5,GUTS6,GUTS7,GUTS8,
GUTS9,GUTS10,GUTS11,GUTS12,


PARTICLE01,PARTICLE02,PARTICLE03,PARTICLE04,
PARTICLE05,PARTICLE06,PARTICLE07,PARTICLE08,
PARTICLE09,PARTICLE10,PARTICLE11,PARTICLE12,

ORGAN01,ORGAN02,ORGAN03,ORGAN04,
ORGAN05,ORGAN06,ORGAN07,ORGAN08,
ORGAN09,ORGAN10,ORGAN11,ORGAN12,

RIB01,RIB02,RIB03,RIB04,
RIB05,RIB06,RIB07,RIB08,
RIB09,RIB10,RIB11,RIB12,

PINK01, PINK02, PINK03, PINK04,
PINK05, PINK06, PINK07, PINK08,
PINK09, PINK10, PINK11, PINK12,

GIBHEAD01,GIBHEAD02,GIBHEAD03,GIBHEAD04,
GIBHEAD05,GIBHEAD06,GIBHEAD07,GIBHEAD08,
GIBHEAD09,GIBHEAD10,GIBHEAD11,GIBHEAD12,

GARM01,GARM02,GARM03,GARM04,
GARM05,GARM06,GARM07,GARM08,
GARM09,GARM10,GARM11,GARM12,

GLEG01,GLEG02,GLEG03,GLEG04,
GLEG05,GLEG06,GLEG07,GLEG08,
GLEG09,GLEG10,GLEG11,GLEG12,

HUMERUS01, HUMERUS02, HUMERUS03, HUMERUS04,
HUMERUS05, HUMERUS06, HUMERUS07, HUMERUS08,
HUMERUS09, HUMERUS10, HUMERUS11, HUMERUS12,

PELVIS01, PELVIS02, PELVIS03, PELVIS04,
PELVIS05, PELVIS06, PELVIS07, PELVIS08,
PELVIS09, PELVIS10, PELVIS11, PELVIS12,

LIMB01, LIMB02, LIMB03, LIMB04,
LIMB05, LIMB06, LIMB07, LIMB08,
LIMB09, LIMB10, LIMB11, LIMB12,


DEADHEAD,

WALLGIB1,WALLGIB2,WALLGIB3,WALLGIB4,

GIBEYE1,GIBEYE2,GIBEYE3,

VAPORIZED1,VAPORIZED2,VAPORIZED3,VAPORIZED4,
VAPORIZED5,VAPORIZED6,VAPORIZED7,VAPORIZED8,

BIGSOUL,LITTLESOUL,
COLLECTOR1,
COLLECTOR3,
COLLECTOR5,
COLLECTOR7,
COLLECTOR9,
COLLECTOR11,
COLLECTOR13,
COLLECTOR15,


ITEMSPAWN1,ITEMSPAWN2,ITEMSPAWN3,ITEMSPAWN4,
ITEMSPAWN5,ITEMSPAWN6,ITEMSPAWN7,ITEMSPAWN8,

DEADBLOOD1,DEADBLOOD2,DEADBLOOD3,DEADBLOOD4,
DEADBLOOD5,DEADBLOOD6,DEADBLOOD7,


FLASH1,FLASH2,FLASH3,FLASH4,
FLASH5,FLASH6,FLASH7,FLASH8,


SKELETON1,SKELETON2,SKELETON3,SKELETON4,
SKELETON5,SKELETON6,SKELETON7,SKELETON8,

SKELETON9,SKELETON10,SKELETON11,SKELETON12,
SKELETON13,SKELETON14,SKELETON15,SKELETON16,

SKELETON17,SKELETON18,SKELETON19,SKELETON20,
SKELETON21,SKELETON22,SKELETON23,SKELETON24,

SKELETON25,SKELETON26,SKELETON27,SKELETON28,
SKELETON29,SKELETON30,SKELETON31,SKELETON32,

SKELETON33,SKELETON34,SKELETON35,SKELETON36,
SKELETON37,SKELETON38,SKELETON39,SKELETON40,

SKELETON41,SKELETON42,SKELETON43,SKELETON44,
SKELETON45,SKELETON46,SKELETON47,SKELETON48,


SPR_PUSHCOLUMN1,SPR_PUSHCOLUMN1a,
/*
SPR_PUSHCOLUMN2,SPR_PUSHCOLUMN2a,
SPR_PUSHCOLUMN3,SPR_PUSHCOLUMN3a,
*/

BLOODSPURT1,BLOODSPURT2,BLOODSPURT3,BLOODSPURT4,
BLOODSPURT5,BLOODSPURT6,BLOODSPURT7,BLOODSPURT8,

GUNSMOKE1,GUNSMOKE2,GUNSMOKE3,GUNSMOKE4,
GUNSMOKE5,GUNSMOKE6,GUNSMOKE7,GUNSMOKE8,



HITMETALWALL1,HITMETALWALL2,HITMETALWALL3,HITMETALWALL4,

HITMETALACTOR1,HITMETALACTOR2,HITMETALACTOR3,HITMETALACTOR4,


PLATFORM1,  PLATFORM2,  PLATFORM3,  PLATFORM4,  PLATFORM5,


SPR_CROSSFIRE11, SPR_CROSSFIRE12,SPR_CROSSFIRE13, SPR_CROSSFIRE14,
SPR_CROSSFIRE15, SPR_CROSSFIRE16,SPR_CROSSFIRE17, SPR_CROSSFIRE18,

SPR_CROSSFIRE31, SPR_CROSSFIRE32,SPR_CROSSFIRE33, SPR_CROSSFIRE34,
SPR_CROSSFIRE35, SPR_CROSSFIRE36,SPR_CROSSFIRE37, SPR_CROSSFIRE38,

SPR_CREXP1,SPR_CREXP2,SPR_CREXP3,SPR_CREXP4,SPR_CREXP5,


GODFIRE1,GODFIRE2,GODFIRE3,GODFIRE4,



SPR_ENFORCER_GR1,SPR_ENFORCER_GR2,SPR_ENFORCER_GR3,SPR_ENFORCER_GR4,
SPR_ENFORCER_GR5,SPR_ENFORCER_GR6,SPR_ENFORCER_GR7,SPR_ENFORCER_GR8,
SPR_ENFORCER_GR9,SPR_ENFORCER_GR10,

SPR_ENFORCER_FALL1,SPR_ENFORCER_FALL2,SPR_ENFORCER_FALL3,SPR_ENFORCER_FALL4,
SPR_ENFORCER_FALL5,SPR_ENFORCER_FALL6,


SPR_MINE1,SPR_MINE2,SPR_MINE3,SPR_MINE4,

SPR_BJMISS11,SPR_BJMISS12,SPR_BJMISS13,SPR_BJMISS14,
SPR_BJMISS15,SPR_BJMISS16,SPR_BJMISS17,SPR_BJMISS18,
SPR_BJMISS19,SPR_BJMISS110,SPR_BJMISS111,SPR_BJMISS112,
SPR_BJMISS113,SPR_BJMISS114,SPR_BJMISS115,SPR_BJMISS116,


SPR_BSTAR1,SPR_BSTAR2,SPR_BSTAR3,SPR_BSTAR4,



GODPOWERUP1,   GODPOWERUP2,   GODPOWERUP3, GODPOWERUP4,
GODPOWERUP5,   GODPOWERUP6,   GODPOWERUP7, GODPOWERUP8,



ELASTICPOWERUP1, ELASTICPOWERUP2, ELASTICPOWERUP3, ELASTICPOWERUP4,
ELASTICPOWERUP5, ELASTICPOWERUP6, ELASTICPOWERUP7, ELASTICPOWERUP8,

RANDOMPOWERUP1,  RANDOMPOWERUP2,  RANDOMPOWERUP3,  RANDOMPOWERUP4,
RANDOMPOWERUP5,  RANDOMPOWERUP6,  RANDOMPOWERUP7,	RANDOMPOWERUP8,

FLEETFEETPOWERUP1,  FLEETFEETPOWERUP2, FLEETFEETPOWERUP3, FLEETFEETPOWERUP4,
FLEETFEETPOWERUP5,  FLEETFEETPOWERUP6, FLEETFEETPOWERUP7, FLEETFEETPOWERUP8,

MUSHROOMPOWERUP1, MUSHROOMPOWERUP2, MUSHROOMPOWERUP3, MUSHROOMPOWERUP4,
MUSHROOMPOWERUP5, MUSHROOMPOWERUP6,	MUSHROOMPOWERUP7, MUSHROOMPOWERUP8,


ONEUP01,       ONEUP02,       ONEUP03,    ONEUP04,
ONEUP05,       ONEUP06,       ONEUP07,		ONEUP08,


LIFEITEMA01,   LIFEITEMA02,   LIFEITEMA03,   LIFEITEMA04,
LIFEITEMA05,   LIFEITEMA06,   LIFEITEMA07,   LIFEITEMA08,

LIFEITEMB01,   LIFEITEMB02,   LIFEITEMB03,   LIFEITEMB04,
LIFEITEMB05,   LIFEITEMB06,   LIFEITEMB07,   LIFEITEMB08,

LIFEITEMC01,   LIFEITEMC02,   LIFEITEMC03,   LIFEITEMC04,
LIFEITEMC05,   LIFEITEMC06,   LIFEITEMC07,   LIFEITEMC08,
LIFEITEMC09,   LIFEITEMC10,   LIFEITEMC11,   LIFEITEMC12,
LIFEITEMC13,   LIFEITEMC14,   LIFEITEMC15,

LIFEITEMD01,   LIFEITEMD02,   LIFEITEMD03,   LIFEITEMD04,
LIFEITEMD05,   LIFEITEMD06,   LIFEITEMD07,   LIFEITEMD08,


RUBBLE1,RUBBLE2,RUBBLE3,RUBBLE4,RUBBLE5,RUBBLE6,RUBBLE7,
RUBBLE8,RUBBLE9,RUBBLE10,

WOODFRAG1,WOODFRAG2,WOODFRAG3,WOODFRAG4,WOODFRAG5,WOODFRAG6,
WOODFRAG7,WOODFRAG8,WOODFRAG9,WOODFRAG10,WOODFRAG11,WOODFRAG12,
WOODFRAG13,WOODFRAG14,



MISSMOKE1,MISSMOKE2,MISSMOKE3,MISSMOKE4,

SPR62_ETOUCH1, SPR63_ETOUCH2,	SPR64_ETOUCH3,

SPR73_GKEY1,   SPR74_GKEY2,   SPR75_GKEY3,   SPR76_GKEY4,
SPR77_GKEY5,   SPR78_GKEY6,   SPR79_GKEY7,   SPR80_GKEY8,
SPR81_GKEY9,   SPR82_GKEY10,  SPR83_GKEY11,  SPR84_GKEY12,
SPR85_GKEY13,  SPR86_GKEY14,  SPR87_GKEY15,  SPR88_GKEY16,


SPR6_GIBS1,    SPR7_GIBS2,	   SPR8_GIBS3,    SPR33_CBARREL,
SPR34_TOUCH1,  SPR35_TOUCH2,  SPR36_TOUCH3,	SPR37_TOUCH4,
SPR40_GARBAGE3,SPR41_SHIT,    BLUEVASE,   SPR54_HAY,
BULLETHOLE,    ALTBULLETHO,
STNPOLE1,      STNPOLE2,      STNPOLE3,      STNPOLE4,
STNPOLE5,      STNPOLE6,      STNPOLE7,      STNPOLE8,


HGRATE1,       HGRATE2,       HGRATE3,
HGRATE4,

SPR0_YLIGHT,   SPR1_RLIGHT,   SPR2_GLIGHT,
SPR3_BLIGHT,	SPR4_CHAND,    SPR5_LAMPOFF, 	SPR5_LAMP,
SPR9_MONKMEAL, SPR_ABRAZIEROFF, SPR_ABRAZIER2,
SPR32_EXPLOS,  SPR38_GARBAGE1,SPR39_GARBAGE2,
SPR42_GRATE,   SPR43_MSHARDS, SPR44_PEDESTAL,SPR45_ETABLE,
SPR46_STOOL,   SPR50_TREE,    SPR51_PLANT,
SPR55_IBARREL, FBASIN1,       FBASIN2,       FBASIN3,
EBASIN,		   PORRIDGE1,     PORRIDGE2,     PORRIDGE3,
PORRIDGE4,	   PORRIDGE5,     PORRIDGE6,		MONKCRYSTAL11,
MONKCRYSTAL12, MONKCRYSTAL13, MONKCRYSTAL14,	MONKCRYSTAL15,
MONKCRYSTAL16, MONKCRYSTAL21,
MONKCRYSTAL22, MONKCRYSTAL23, MONKCRYSTAL24, MONKCRYSTAL25,
MONKCRYSTAL26, MONKCRYSTAL27, TORCH1,
TORCH2,        TORCH3,        TORCH4,		   TORCH5,
TORCH6,        TORCH7,        TORCH8,		   TORCH9,
TORCH10,       TORCH11,       TORCH12,		   TORCH13,
TORCH14,       TORCH15,

/*
ATORCH2,       ATORCH3,
ATORCH4,		   ATORCH5,       ATORCH6,       ATORCH7,
ATORCH8,		   ATORCH9,       ATORCH10,      ATORCH11,
ATORCH12,		ATORCH13,		ATORCH14,      ATORCH15,
ATORCH16,
*/

FFLAME1,
FFLAME2,       FFLAME3,       FFLAME4,		   FFLAME5,
FFLAME6,       FFLAME7,       SPR_TWOPIST,   SPR_MP40,
SPR_BAZOOKA,   SPR_FIREBOMB,	SPR_HEATSEEK,  SPR_DRUNK,
SPR_FIREWALL,  SPR_KES,       SPR_SPLIT,



GASMASKPOWERUP,
BULLETPROOFPOWERUP, ASBESTOSPOWERUP,PREPIT,POSTPIT,


CASSATT_SHOOT11,CASSATT_SHOOT12,CASSATT_SHOOT13,CASSATT_SHOOT14,
CASSATT_SHOOT15,CASSATT_SHOOT16,CASSATT_SHOOT17,CASSATT_SHOOT18,

CASSATT_SHOOT21,CASSATT_SHOOT22,CASSATT_SHOOT23,CASSATT_SHOOT24,
CASSATT_SHOOT25,CASSATT_SHOOT26,CASSATT_SHOOT27,CASSATT_SHOOT28,



CASSATTM_SHOOT11,CASSATTM_SHOOT12,CASSATTM_SHOOT13,CASSATTM_SHOOT14,
CASSATTM_SHOOT15,CASSATTM_SHOOT16,CASSATTM_SHOOT17,CASSATTM_SHOOT18,

CASSATTM_SHOOT21,CASSATTM_SHOOT22,CASSATTM_SHOOT23,CASSATTM_SHOOT24,
CASSATTM_SHOOT25,CASSATTM_SHOOT26,CASSATTM_SHOOT27,CASSATTM_SHOOT28,



CASSATT_S1,CASSATT_S2,CASSATT_S3,CASSATT_S4,
CASSATT_S5,CASSATT_S6,CASSATT_S7,CASSATT_S8,

CASSATT_W11,CASSATT_W12,CASSATT_W13,CASSATT_W14,
CASSATT_W15,CASSATT_W16,CASSATT_W17,CASSATT_W18,

CASSATT_W21,CASSATT_W22,CASSATT_W23,CASSATT_W24,
CASSATT_W25,CASSATT_W26,CASSATT_W27,CASSATT_W28,

CASSATT_W31,CASSATT_W32,CASSATT_W33,CASSATT_W34,
CASSATT_W35,CASSATT_W36,CASSATT_W37,CASSATT_W38,

CASSATT_W41,CASSATT_W42,CASSATT_W43,CASSATT_W44,
CASSATT_W45,CASSATT_W46,CASSATT_W47,CASSATT_W48,


CASSATT_VDIE1,CASSATT_VDIE2,CASSATT_VDIE3,CASSATT_VDIE4,CASSATT_VDIE5,
CASSATT_VDIE6,
CASSATT_VDEAD,

CASSATT_DIE1,CASSATT_DIE2,CASSATT_DIE3,CASSATT_DIE4,CASSATT_DIE5,
CASSATT_DIE6,
CASSATT_DEAD,


// DDOI - Wad ends here

BARRETT_SHOOT11,BARRETT_SHOOT12,BARRETT_SHOOT13,BARRETT_SHOOT14,
BARRETT_SHOOT15,BARRETT_SHOOT16,BARRETT_SHOOT17,BARRETT_SHOOT18,


BARRETT_SHOOT21,BARRETT_SHOOT22,BARRETT_SHOOT23,BARRETT_SHOOT24,
BARRETT_SHOOT25,BARRETT_SHOOT26,BARRETT_SHOOT27,BARRETT_SHOOT28,


BARRETTM_SHOOT11,BARRETTM_SHOOT12,BARRETTM_SHOOT13,BARRETTM_SHOOT14,
BARRETTM_SHOOT15,BARRETTM_SHOOT16,BARRETTM_SHOOT17,BARRETTM_SHOOT18,

BARRETTM_SHOOT21,BARRETTM_SHOOT22,BARRETTM_SHOOT23,BARRETTM_SHOOT24,
BARRETTM_SHOOT25,BARRETTM_SHOOT26,BARRETTM_SHOOT27,BARRETTM_SHOOT28,



BARRETT_S1,BARRETT_S2,BARRETT_S3,BARRETT_S4,
BARRETT_S5,BARRETT_S6,BARRETT_S7,BARRETT_S8,

BARRETT_W11,BARRETT_W12,BARRETT_W13,BARRETT_W14,
BARRETT_W15,BARRETT_W16,BARRETT_W17,BARRETT_W18,

BARRETT_W21,BARRETT_W22,BARRETT_W23,BARRETT_W24,
BARRETT_W25,BARRETT_W26,BARRETT_W27,BARRETT_W28,

BARRETT_W31,BARRETT_W32,BARRETT_W33,BARRETT_W34,
BARRETT_W35,BARRETT_W36,BARRETT_W37,BARRETT_W38,

BARRETT_W41,BARRETT_W42,BARRETT_W43,BARRETT_W44,
BARRETT_W45,BARRETT_W46,BARRETT_W47,BARRETT_W48,


BARRETT_VDIE1,BARRETT_VDIE2,BARRETT_VDIE3,BARRETT_VDIE4,BARRETT_VDIE5,
BARRETT_VDIE6,
BARRETT_VDEAD,

BARRETT_DIE1,BARRETT_DIE2,BARRETT_DIE3,BARRETT_DIE4,BARRETT_DIE5,
BARRETT_DIE6,
BARRETT_DEAD,



WENDT_SHOOT11,WENDT_SHOOT12,WENDT_SHOOT13,WENDT_SHOOT14,
WENDT_SHOOT15,WENDT_SHOOT16,WENDT_SHOOT17,WENDT_SHOOT18,

WENDT_SHOOT21,WENDT_SHOOT22,WENDT_SHOOT23,WENDT_SHOOT24,
WENDT_SHOOT25,WENDT_SHOOT26,WENDT_SHOOT27,WENDT_SHOOT28,



WENDTM_SHOOT11,WENDTM_SHOOT12,WENDTM_SHOOT13,WENDTM_SHOOT14,
WENDTM_SHOOT15,WENDTM_SHOOT16,WENDTM_SHOOT17,WENDTM_SHOOT18,

WENDTM_SHOOT21,WENDTM_SHOOT22,WENDTM_SHOOT23,WENDTM_SHOOT24,
WENDTM_SHOOT25,WENDTM_SHOOT26,WENDTM_SHOOT27,WENDTM_SHOOT28,



WENDT_S1,WENDT_S2,WENDT_S3,WENDT_S4,
WENDT_S5,WENDT_S6,WENDT_S7,WENDT_S8,

WENDT_W11,WENDT_W12,WENDT_W13,WENDT_W14,
WENDT_W15,WENDT_W16,WENDT_W17,WENDT_W18,

WENDT_W21,WENDT_W22,WENDT_W23,WENDT_W24,
WENDT_W25,WENDT_W26,WENDT_W27,WENDT_W28,

WENDT_W31,WENDT_W32,WENDT_W33,WENDT_W34,
WENDT_W35,WENDT_W36,WENDT_W37,WENDT_W38,

WENDT_W41,WENDT_W42,WENDT_W43,WENDT_W44,
WENDT_W45,WENDT_W46,WENDT_W47,WENDT_W48,


WENDT_VDIE1,WENDT_VDIE2,WENDT_VDIE3,WENDT_VDIE4,WENDT_VDIE5,WENDT_VDIE6,
WENDT_VDEAD,

WENDT_DIE1,WENDT_DIE2,WENDT_DIE3,WENDT_DIE4,WENDT_DIE5,WENDT_DIE6,
WENDT_DEAD,


//======


NI_SHOOT11,NI_SHOOT12,NI_SHOOT13,NI_SHOOT14,
NI_SHOOT15,NI_SHOOT16,NI_SHOOT17,NI_SHOOT18,

NI_SHOOT21,NI_SHOOT22,NI_SHOOT23,NI_SHOOT24,
NI_SHOOT25,NI_SHOOT26,NI_SHOOT27,NI_SHOOT28,



NIM_SHOOT11,NIM_SHOOT12,NIM_SHOOT13,NIM_SHOOT14,
NIM_SHOOT15,NIM_SHOOT16,NIM_SHOOT17,NIM_SHOOT18,

NIM_SHOOT21,NIM_SHOOT22,NIM_SHOOT23,NIM_SHOOT24,
NIM_SHOOT25,NIM_SHOOT26,NIM_SHOOT27,NIM_SHOOT28,



NI_S1,NI_S2,NI_S3,NI_S4,
NI_S5,NI_S6,NI_S7,NI_S8,

NI_W11,NI_W12,NI_W13,NI_W14,
NI_W15,NI_W16,NI_W17,NI_W18,

NI_W21,NI_W22,NI_W23,NI_W24,
NI_W25,NI_W26,NI_W27,NI_W28,

NI_W31,NI_W32,NI_W33,NI_W34,
NI_W35,NI_W36,NI_W37,NI_W38,

NI_W41,NI_W42,NI_W43,NI_W44,
NI_W45,NI_W46,NI_W47,NI_W48,


NI_VDIE1,NI_VDIE2,NI_VDIE3,NI_VDIE4,NI_VDIE5,NI_VDIE6,
NI_VDEAD,

NI_DIE1,NI_DIE2,NI_DIE3,NI_DIE4,NI_DIE5,NI_DIE6,
NI_DEAD,




IPF_SHOOT11,IPF_SHOOT12,IPF_SHOOT13,IPF_SHOOT14,
IPF_SHOOT15,IPF_SHOOT16,IPF_SHOOT17,IPF_SHOOT18,

IPF_SHOOT21,IPF_SHOOT22,IPF_SHOOT23,IPF_SHOOT24,
IPF_SHOOT25,IPF_SHOOT26,IPF_SHOOT27,IPF_SHOOT28,


IPFM_SHOOT11,IPFM_SHOOT12,IPFM_SHOOT13,IPFM_SHOOT14,
IPFM_SHOOT15,IPFM_SHOOT16,IPFM_SHOOT17,IPFM_SHOOT18,

IPFM_SHOOT21,IPFM_SHOOT22,IPFM_SHOOT23,IPFM_SHOOT24,
IPFM_SHOOT25,IPFM_SHOOT26,IPFM_SHOOT27,IPFM_SHOOT28,



IPF_S1,IPF_S2,IPF_S3,IPF_S4,
IPF_S5,IPF_S6,IPF_S7,IPF_S8,

IPF_W11,IPF_W12,IPF_W13,IPF_W14,
IPF_W15,IPF_W16,IPF_W17,IPF_W18,

IPF_W21,IPF_W22,IPF_W23,IPF_W24,
IPF_W25,IPF_W26,IPF_W27,IPF_W28,

IPF_W31,IPF_W32,IPF_W33,IPF_W34,
IPF_W35,IPF_W36,IPF_W37,IPF_W38,

IPF_W41,IPF_W42,IPF_W43,IPF_W44,
IPF_W45,IPF_W46,IPF_W47,IPF_W48,


IPF_VDIE1,IPF_VDIE2,IPF_VDIE3,IPF_VDIE4,IPF_VDIE5,IPF_VDIE6,
IPF_VDEAD,

IPF_DIE1,IPF_DIE2,IPF_DIE3,IPF_DIE4,IPF_DIE5,IPF_DIE6,
IPF_DEAD,


SERIALDOG_W11,SERIALDOG_W12,SERIALDOG_W13,SERIALDOG_W14,
SERIALDOG_W15,SERIALDOG_W16,SERIALDOG_W17,SERIALDOG_W18,

SERIALDOG_W21,SERIALDOG_W22,SERIALDOG_W23,SERIALDOG_W24,
SERIALDOG_W25,SERIALDOG_W26,SERIALDOG_W27,SERIALDOG_W28,

SERIALDOG_W31,SERIALDOG_W32,SERIALDOG_W33,SERIALDOG_W34,
SERIALDOG_W35,SERIALDOG_W36,SERIALDOG_W37,SERIALDOG_W38,

SERIALDOG_W41,SERIALDOG_W42,SERIALDOG_W43,SERIALDOG_W44,
SERIALDOG_W45,SERIALDOG_W46,SERIALDOG_W47,SERIALDOG_W48,

SERIALDOG_ATTACK1,SERIALDOG_ATTACK2,SERIALDOG_ATTACK3,SERIALDOG_ATTACK4,
SERIALDOG_ATTACK5,SERIALDOG_ATTACK6,SERIALDOG_ATTACK7,SERIALDOG_ATTACK8,


/***************   Lowguard 2 (Marianna)  *************************************/


/*
SPR_MARIANNA_SHOOT1,SPR_MARIANNA_SHOOT2,SPR_MARIANNA_SHOOT3,SPR_MARIANNA_SHOOT4,

SPR_MARIANNA_S1,SPR_MARIANNA_S2,SPR_MARIANNA_S3,SPR_MARIANNA_S4,
SPR_MARIANNA_S5,SPR_MARIANNA_S6,SPR_MARIANNA_S7,SPR_MARIANNA_S8,

SPR_MARIANNA_W11,SPR_MARIANNA_W12,SPR_MARIANNA_W13,SPR_MARIANNA_W14,
SPR_MARIANNA_W15,SPR_MARIANNA_W16,SPR_MARIANNA_W17,SPR_MARIANNA_W18,

SPR_MARIANNA_W21,SPR_MARIANNA_W22,SPR_MARIANNA_W23,SPR_MARIANNA_W24,
SPR_MARIANNA_W25,SPR_MARIANNA_W26,SPR_MARIANNA_W27,SPR_MARIANNA_W28,

SPR_MARIANNA_W31,SPR_MARIANNA_W32,SPR_MARIANNA_W33,SPR_MARIANNA_W34,
SPR_MARIANNA_W35,SPR_MARIANNA_W36,SPR_MARIANNA_W37,SPR_MARIANNA_W38,

SPR_MARIANNA_W41,SPR_MARIANNA_W42,SPR_MARIANNA_W43,SPR_MARIANNA_W44,
SPR_MARIANNA_W45,SPR_MARIANNA_W46,SPR_MARIANNA_W47,SPR_MARIANNA_W48,


SPR_MARIANNA_PAIN1, SPR_MARIANNA_PAIN2, SPR_MARIANNA_DIE1, SPR_MARIANNA_DIE2,
SPR_MARIANNA_DIE3, SPR_MARIANNA_DIE4, SPR_MARIANNA_DEAD,

SPR_MARIANNA_WPAIN1, SPR_MARIANNA_WPAIN2, SPR_MARIANNA_WDIE1, SPR_MARIANNA_WDIE2,
SPR_MARIANNA_WDIE3, SPR_MARIANNA_WDIE4, SPR_MARIANNA_WDEAD,


SPR_SNMAR_DEAD, SPR_MARISE1, SPR_MARISE2, SPR_MARISE3, SPR_MARISE4,
*/


/****************** OBERPATROLLE (Nolan) ***********************************/


SPR_OP_SHOOT1,SPR_OP_SHOOT2,SPR_OP_SHOOT3,SPR_OP_SHOOT4,

SPR_OP_BOLOSHOOT1,SPR_OP_BOLOSHOOT2,SPR_OP_BOLOSHOOT3,SPR_OP_BOLOSHOOT4,
SPR_OP_BOLOSHOOT5,

SPR_OP_S1,SPR_OP_S2,SPR_OP_S3,SPR_OP_S4,
SPR_OP_S5,SPR_OP_S6,SPR_OP_S7,SPR_OP_S8,

SPR_OP_W11,SPR_OP_W12,SPR_OP_W13,SPR_OP_W14,
SPR_OP_W15,SPR_OP_W16,SPR_OP_W17,SPR_OP_W18,

SPR_OP_W21,SPR_OP_W22,SPR_OP_W23,SPR_OP_W24,
SPR_OP_W25,SPR_OP_W26,SPR_OP_W27,SPR_OP_W28,

SPR_OP_W31,SPR_OP_W32,SPR_OP_W33,SPR_OP_W34,
SPR_OP_W35,SPR_OP_W36,SPR_OP_W37,SPR_OP_W38,

SPR_OP_W41,SPR_OP_W42,SPR_OP_W43,SPR_OP_W44,
SPR_OP_W45,SPR_OP_W46,SPR_OP_W47,SPR_OP_W48,


SPR_OP_PAIN1,SPR_OP_PAIN2,
SPR_OP_ALTDIE1,SPR_OP_ALTDIE2,SPR_OP_ALTDIE3,
SPR_OP_ALTDIE4,SPR_OP_ALTDIE5,SPR_OP_ALTDEAD,

SPR_OP_WPAIN1,SPR_OP_WPAIN2,
SPR_OP_WALTDIE1,SPR_OP_WALTDIE2,SPR_OP_WALTDIE3,
SPR_OP_WALTDIE4,SPR_OP_WALTDIE5,SPR_OP_WALTDEAD,

SPR_OP_DIE1,SPR_OP_DIE2,SPR_OP_DIE3,
SPR_OP_DIE4,SPR_OP_DIE5,SPR_OP_DIE6,
SPR_OP_DEAD,





/******************* Normal monk (Lee)**********************************/


SPR_MONK_DRAIN1,SPR_MONK_DRAIN2,SPR_MONK_DRAIN3,SPR_MONK_DRAIN4,
SPR_MONK_DRAIN5,SPR_MONK_DRAIN6,

/*SPR_MONK_CAST1,SPR_MONK_CAST2,SPR_MONK_CAST3,SPR_MONK_CAST4,
SPR_MONK_CAST5,SPR_MONK_CAST6,SPR_MONK_CAST7,SPR_MONK_CAST8,*/

SPR_MONK_S1,SPR_MONK_S2,SPR_MONK_S3,SPR_MONK_S4,
SPR_MONK_S5,SPR_MONK_S6,SPR_MONK_S7,SPR_MONK_S8,

SPR_MONK_W11,SPR_MONK_W12,SPR_MONK_W13,SPR_MONK_W14,
SPR_MONK_W15,SPR_MONK_W16,SPR_MONK_W17,SPR_MONK_W18,

SPR_MONK_W21,SPR_MONK_W22,SPR_MONK_W23,SPR_MONK_W24,
SPR_MONK_W25,SPR_MONK_W26,SPR_MONK_W27,SPR_MONK_W28,

SPR_MONK_W31,SPR_MONK_W32,SPR_MONK_W33,SPR_MONK_W34,
SPR_MONK_W35,SPR_MONK_W36,SPR_MONK_W37,SPR_MONK_W38,

SPR_MONK_W41,SPR_MONK_W42,SPR_MONK_W43,SPR_MONK_W44,
SPR_MONK_W45,SPR_MONK_W46,SPR_MONK_W47,SPR_MONK_W48,

SPR_MONK_PAIN1,SPR_MONK_PAIN2,

SPR_MONK_DIE1,SPR_MONK_DIE2,SPR_MONK_DIE3,SPR_MONK_DIE4,
SPR_MONK_DEAD,

/*SPR_MONK_USE11,SPR_MONK_USE12,SPR_MONK_USE13,SPR_MONK_USE14,
SPR_MONK_USE15,SPR_MONK_USE16,SPR_MONK_USE17,SPR_MONK_USE18,

SPR_MONK_USE21,SPR_MONK_USE22,SPR_MONK_USE23,SPR_MONK_USE24,
SPR_MONK_USE25,SPR_MONK_USE26,SPR_MONK_USE27,SPR_MONK_USE28,
*/


/******************* Fire monk (Allen)**********************************/

SPR_FIREMONK_CAST1,SPR_FIREMONK_CAST2,SPR_FIREMONK_CAST3,SPR_FIREMONK_CAST4,
SPR_FIREMONK_CAST5,SPR_FIREMONK_CAST6,SPR_FIREMONK_CAST7,

SPR_FIREMONK_S1,SPR_FIREMONK_S2,SPR_FIREMONK_S3,SPR_FIREMONK_S4,
SPR_FIREMONK_S5,SPR_FIREMONK_S6,SPR_FIREMONK_S7,SPR_FIREMONK_S8,

SPR_FIREMONK_W11,SPR_FIREMONK_W12,SPR_FIREMONK_W13,SPR_FIREMONK_W14,
SPR_FIREMONK_W15,SPR_FIREMONK_W16,SPR_FIREMONK_W17,SPR_FIREMONK_W18,

SPR_FIREMONK_W21,SPR_FIREMONK_W22,SPR_FIREMONK_W23,SPR_FIREMONK_W24,
SPR_FIREMONK_W25,SPR_FIREMONK_W26,SPR_FIREMONK_W27,SPR_FIREMONK_W28,

SPR_FIREMONK_W31,SPR_FIREMONK_W32,SPR_FIREMONK_W33,SPR_FIREMONK_W34,
SPR_FIREMONK_W35,SPR_FIREMONK_W36,SPR_FIREMONK_W37,SPR_FIREMONK_W38,

SPR_FIREMONK_W41,SPR_FIREMONK_W42,SPR_FIREMONK_W43,SPR_FIREMONK_W44,
SPR_FIREMONK_W45,SPR_FIREMONK_W46,SPR_FIREMONK_W47,SPR_FIREMONK_W48,

SPR_FIREMONK_PAIN1,SPR_FIREMONK_PAIN2,

SPR_FIREMONK_DIE1,SPR_FIREMONK_DIE2,SPR_FIREMONK_DIE3,SPR_FIREMONK_DIE4,
SPR_FIREMONK_DEAD1,SPR_FIREMONK_DEAD2,SPR_FIREMONK_DEAD3,SPR_FIREMONK_DEAD4,
SPR_FIREMONK_DEAD5,SPR_FIREMONK_DEAD6,SPR_FIREMONK_DEAD7,



/*
SPR_ALLEN_USE11,SPR_ALLEN_USE12,SPR_ALLEN_USE13,SPR_ALLEN_USE14,
SPR_ALLEN_USE15,SPR_ALLEN_USE16,SPR_ALLEN_USE17,SPR_ALLEN_USE18,

SPR_ALLEN_USE21,SPR_ALLEN_USE22,SPR_ALLEN_USE23,SPR_ALLEN_USE24,
SPR_ALLEN_USE25,SPR_ALLEN_USE26,SPR_ALLEN_USE27,SPR_ALLEN_USE28,
*/
BCRAFT01,BCRAFT02,BCRAFT03,BCRAFT04,
BCRAFT05,BCRAFT06,BCRAFT07,BCRAFT08,
BCRAFT09,BCRAFT10,BCRAFT11,BCRAFT12,
BCRAFT13,BCRAFT14,BCRAFT15,BCRAFT16,


/************************* GENERAL DARIAN *******************************/


SPR_DARIAN_SHOOT1,SPR_DARIAN_SHOOT2,SPR_DARIAN_SHOOT3,SPR_DARIAN_SHOOT4,

SPR_DARIAN_SINK1,SPR_DARIAN_SINK2,SPR_DARIAN_SINK3,SPR_DARIAN_SINK4,
SPR_DARIAN_SINK5,SPR_DARIAN_SINK6,SPR_DARIAN_SINK7,SPR_DARIAN_SINK8,


SPR_DARIAN_S1,SPR_DARIAN_S2,SPR_DARIAN_S3,SPR_DARIAN_S4,
SPR_DARIAN_S5,SPR_DARIAN_S6,SPR_DARIAN_S7,SPR_DARIAN_S8,

SPR_DARIAN_W11,SPR_DARIAN_W12,SPR_DARIAN_W13,SPR_DARIAN_W14,
SPR_DARIAN_W15,SPR_DARIAN_W16,SPR_DARIAN_W17,SPR_DARIAN_W18,

SPR_DARIAN_W21,SPR_DARIAN_W22,SPR_DARIAN_W23,SPR_DARIAN_W24,
SPR_DARIAN_W25,SPR_DARIAN_W26,SPR_DARIAN_W27,SPR_DARIAN_W28,

SPR_DARIAN_W31,SPR_DARIAN_W32,SPR_DARIAN_W33,SPR_DARIAN_W34,
SPR_DARIAN_W35,SPR_DARIAN_W36,SPR_DARIAN_W37,SPR_DARIAN_W38,

SPR_DARIAN_W41,SPR_DARIAN_W42,SPR_DARIAN_W43,SPR_DARIAN_W44,
SPR_DARIAN_W45,SPR_DARIAN_W46,SPR_DARIAN_W47,SPR_DARIAN_W48,


SPR_DARIAN_PAIN1, SPR_DARIAN_PAIN2, SPR_DARIAN_DIE1, SPR_DARIAN_DIE2,
SPR_DARIAN_DIE3, SPR_DARIAN_DIE4,SPR_DARIAN_DIE5, SPR_DARIAN_DIE6,
SPR_DARIAN_DIE7, SPR_DARIAN_DIE8,SPR_DARIAN_DIE9, SPR_DARIAN_DIE10,
SPR_DARIAN_DEAD,

SPR_DARIAN_WPAIN1, SPR_DARIAN_WPAIN2, SPR_WDARIAN_DIE1, SPR_WDARIAN_DIE2,
SPR_DARIAN_WDIE3, SPR_DARIAN_WDIE4,SPR_WDARIAN_DIE5, SPR_WDARIAN_DIE6,
SPR_DARIAN_WDIE7, SPR_DARIAN_WDIE8,SPR_WDARIAN_DIE9, SPR_WDARIAN_DIE10,
SPR_DARIAN_WDEAD,


SPR_DARIAN_USE11,SPR_DARIAN_USE12,SPR_DARIAN_USE13,SPR_DARIAN_USE14,
SPR_DARIAN_USE15,SPR_DARIAN_USE16,SPR_DARIAN_USE17,SPR_DARIAN_USE18,

SPR_DARIAN_USE21,SPR_DARIAN_USE22,SPR_DARIAN_USE23,SPR_DARIAN_USE24,
SPR_DARIAN_USE25,SPR_DARIAN_USE26,SPR_DARIAN_USE27,SPR_DARIAN_USE28,



/************************* HEINRICH KRIST ******************************/


SPR_KRIST_S1,SPR_KRIST_S2,SPR_KRIST_S3,SPR_KRIST_S4,
SPR_KRIST_S5,SPR_KRIST_S6,SPR_KRIST_S7,SPR_KRIST_S8,

SPR_KRIST_LEFT1,SPR_KRIST_LEFT2,SPR_KRIST_LEFT3,SPR_KRIST_LEFT4,
SPR_KRIST_LEFT5,SPR_KRIST_LEFT6,SPR_KRIST_LEFT7,SPR_KRIST_LEFT8,


SPR_KRIST_SHOOT1,SPR_KRIST_SHOOT2,SPR_KRIST_SHOOT3,SPR_KRIST_SHOOT4,
SPR_KRIST_SHOOT5,SPR_KRIST_SHOOT6,SPR_KRIST_SHOOT7,SPR_KRIST_SHOOT8,
SPR_KRIST_SHOOT9,SPR_KRIST_SHOOT10,SPR_KRIST_SHOOT11,

//SPR_KRIST_PAIN1,SPR_KRIST_PAIN2,SPR_KRIST_PAIN3,SPR_KRIST_PAIN4,
//SPR_KRIST_PAIN5,SPR_KRIST_PAIN6,SPR_KRIST_PAIN7,SPR_KRIST_PAIN8,

SPR_KRIST_DIE1,SPR_KRIST_DIE2,SPR_KRIST_DEAD1,SPR_KRIST_DEAD2,
SPR_KRIST_DEAD3,SPR_KRIST_DEAD4,SPR_KRIST_DEAD5,SPR_KRIST_DEAD6,
SPR_KRIST_DEAD7,SPR_KRIST_DEAD8,

SPR_KRIST_RIGHT1,SPR_KRIST_RIGHT2,SPR_KRIST_RIGHT3,SPR_KRIST_RIGHT4,
SPR_KRIST_RIGHT5,SPR_KRIST_RIGHT6,SPR_KRIST_RIGHT7,SPR_KRIST_RIGHT8,

SPR_KRIST_FOR1,SPR_KRIST_FOR2,SPR_KRIST_FOR3,SPR_KRIST_FOR4,
SPR_KRIST_FOR5,SPR_KRIST_FOR6,SPR_KRIST_FOR7,SPR_KRIST_FOR8,

//SPR_KRIST_BACK1,SPR_KRIST_BACK2,SPR_KRIST_BACK3,SPR_KRIST_BACK4,
//SPR_KRIST_BACK5,SPR_KRIST_BACK6,SPR_KRIST_BACK7,SPR_KRIST_BACK8,

SPR_KRIST_MINERIGHT1,SPR_KRIST_MINERIGHT2,SPR_KRIST_MINERIGHT3,SPR_KRIST_MINERIGHT4,
SPR_KRIST_MINERIGHT5,SPR_KRIST_MINERIGHT6,SPR_KRIST_MINERIGHT7,SPR_KRIST_MINERIGHT8,

//SPR_KRIST_MINELEFT1,SPR_KRIST_MINELEFT2,SPR_KRIST_MINELEFT3,SPR_KRIST_MINELEFT4,
//SPR_KRIST_MINELEFT5,SPR_KRIST_MINELEFT6,SPR_KRIST_MINELEFT7,SPR_KRIST_MINELEFT8,

SPR_KRIST_DOPE1,SPR_KRIST_DOPE2,SPR_KRIST_DOPE3,


/************************* NME  ******************************/

NMEHEAD1_01,NMEHEAD1_02,NMEHEAD1_03,NMEHEAD1_04,
NMEHEAD1_05,NMEHEAD1_06,NMEHEAD1_07,NMEHEAD1_08,
NMEHEAD1_09,NMEHEAD1_10,NMEHEAD1_11,NMEHEAD1_12,
NMEHEAD1_13,NMEHEAD1_14,NMEHEAD1_15,NMEHEAD1_16,

NMEHEAD2_01,NMEHEAD2_02,NMEHEAD2_03,NMEHEAD2_04,
NMEHEAD2_05,NMEHEAD2_06,NMEHEAD2_07,NMEHEAD2_08,
NMEHEAD2_09,NMEHEAD2_10,NMEHEAD2_11,NMEHEAD2_12,
NMEHEAD2_13,NMEHEAD2_14,NMEHEAD2_15,NMEHEAD2_16,

NMEBODY1_01,NMEBODY1_02,NMEBODY1_03,NMEBODY1_04,
NMEBODY1_05,NMEBODY1_06,NMEBODY1_07,NMEBODY1_08,
NMEBODY1_09,NMEBODY1_10,NMEBODY1_11,NMEBODY1_12,
NMEBODY1_13,NMEBODY1_14,NMEBODY1_15,NMEBODY1_16,

NMEBODY2_01,NMEBODY2_02,NMEBODY2_03,NMEBODY2_04,
NMEBODY2_05,NMEBODY2_06,NMEBODY2_07,NMEBODY2_08,
NMEBODY2_09,NMEBODY2_10,NMEBODY2_11,NMEBODY2_12,
NMEBODY2_13,NMEBODY2_14,NMEBODY2_15,NMEBODY2_16,

NMEBODY3_01,NMEBODY3_02,NMEBODY3_03,NMEBODY3_04,
NMEBODY3_05,NMEBODY3_06,NMEBODY3_07,NMEBODY3_08,
NMEBODY3_09,NMEBODY3_10,NMEBODY3_11,NMEBODY3_12,
NMEBODY3_13,NMEBODY3_14,NMEBODY3_15,NMEBODY3_16,

NMEBODY4_01,NMEBODY4_02,NMEBODY4_03,NMEBODY4_04,
NMEBODY4_05,NMEBODY4_06,NMEBODY4_07,NMEBODY4_08,
NMEBODY4_09,NMEBODY4_10,NMEBODY4_11,NMEBODY4_12,
NMEBODY4_13,NMEBODY4_14,NMEBODY4_15,NMEBODY4_16,

NMEWHEEL1_01,NMEWHEEL1_02,NMEWHEEL1_03,NMEWHEEL1_04,
NMEWHEEL1_05,NMEWHEEL1_06,NMEWHEEL1_07,NMEWHEEL1_08,
NMEWHEEL1_09,NMEWHEEL1_10,NMEWHEEL1_11,NMEWHEEL1_12,
NMEWHEEL1_13,NMEWHEEL1_14,NMEWHEEL1_15,NMEWHEEL1_16,

NMEWHEEL2_01,NMEWHEEL2_02,NMEWHEEL2_03,NMEWHEEL2_04,
NMEWHEEL2_05,NMEWHEEL2_06,NMEWHEEL2_07,NMEWHEEL2_08,
NMEWHEEL2_09,NMEWHEEL2_10,NMEWHEEL2_11,NMEWHEEL2_12,
NMEWHEEL2_13,NMEWHEEL2_14,NMEWHEEL2_15,NMEWHEEL2_16,

NMEWHEEL3_01,NMEWHEEL3_02,NMEWHEEL3_03,NMEWHEEL3_04,
NMEWHEEL3_05,NMEWHEEL3_06,NMEWHEEL3_07,NMEWHEEL3_08,
NMEWHEEL3_09,NMEWHEEL3_10,NMEWHEEL3_11,NMEWHEEL3_12,
NMEWHEEL3_13,NMEWHEEL3_14,NMEWHEEL3_15,NMEWHEEL3_16,

NMEWHEEL4_01,NMEWHEEL4_02,NMEWHEEL4_03,NMEWHEEL4_04,
NMEWHEEL4_05,NMEWHEEL4_06,NMEWHEEL4_07,NMEWHEEL4_08,
NMEWHEEL4_09,NMEWHEEL4_10,NMEWHEEL4_11,NMEWHEEL4_12,
NMEWHEEL4_13,NMEWHEEL4_14,NMEWHEEL4_15,NMEWHEEL4_16,

NMEWHEEL5_01,NMEWHEEL5_02,NMEWHEEL5_03,NMEWHEEL5_04,
NMEWHEEL5_05,NMEWHEEL5_06,NMEWHEEL5_07,NMEWHEEL5_08,
NMEWHEEL5_09,NMEWHEEL5_10,NMEWHEEL5_11,NMEWHEEL5_12,
NMEWHEEL5_13,NMEWHEEL5_14,NMEWHEEL5_15,NMEWHEEL5_16,

NMEROCKET_01,NMEROCKET_02,NMEROCKET_03,NMEROCKET_04,
NMEROCKET_05,NMEROCKET_06,NMEROCKET_07,NMEROCKET_08,
NMEROCKET_09,NMEROCKET_10,NMEROCKET_11,NMEROCKET_12,
NMEROCKET_13,NMEROCKET_14,NMEROCKET_15,NMEROCKET_16,

NMEROCKET2_01,NMEROCKET2_02,NMEROCKET2_03,NMEROCKET2_04,
NMEROCKET2_05,NMEROCKET2_06,NMEROCKET2_07,NMEROCKET2_08,
NMEROCKET2_09,NMEROCKET2_10,NMEROCKET2_11,NMEROCKET2_12,
NMEROCKET2_13,NMEROCKET2_14,NMEROCKET2_15,NMEROCKET2_16,

NMEMINIBALL_01,NMEMINIBALL_02,NMEMINIBALL_03,NMEMINIBALL_04,
NMESAUCER_01,NMESAUCER_02,NMESAUCER_03,NMESAUCER_04,


/************************** TOM ****************************************/

  TOMS1,TOMS2,TOMS3,TOMS4,TOMS5,TOMS6,TOMS7,TOMS8,

  TOMFLY11,TOMFLY12,TOMFLY13,TOMFLY14,
  TOMFLY15,TOMFLY16,TOMFLY17,TOMFLY18,

  TOMFLY21,TOMFLY22,TOMFLY23,TOMFLY24,
  TOMFLY25,TOMFLY26,TOMFLY27,TOMFLY28,

  TOMLG1,TOMLG2,TOMLG3,TOMLG4,
  TOMLG5,TOMLG6,TOMLG7,TOMLG8,

  TOMLG9,TOMLG10,TOMLG11,

  TOMFS1,TOMFS2,TOMFS3,TOMFS4,
  TOMFS5,TOMFS6,

  TOMBR1,TOMBR2,TOMBR3,TOMBR4,
  TOMBR5,TOMBR6,TOMBR7,TOMBR8,

  TOMHEAD1,TOMHEAD2,TOMHEAD3,TOMHEAD4,
  TOMHEAD5,TOMHEAD6,TOMHEAD7,TOMHEAD8,

  TPREPARE ,
  THDIE1  ,
  THDIE2  ,
  TAWAKEN1,TAWAKEN2,TAWAKEN3,TAWAKEN4,
  TAWAKEN5,

  THBALL1 ,THBALL2 ,THBALL3 ,THBALL4 ,
  THBALL5 ,THBALL6 ,THBALL7 ,THBALL8 ,
  THBALL9 ,

  TSPHERE1,TSPHERE2,TSPHERE3,TSPHERE4,
  TSPHERE5,TSPHERE6,TSPHERE7,TSPHERE8,
  TSPHERE9,TSPHERE10,

  TBBALL1 ,TBBALL2 ,TBBALL3 ,TBBALL4 ,
  TBBALL5 ,TBBALL6 ,TBBALL7 ,TBBALL8 ,
  TBBALL9 ,

  TSCAREB1,TSCAREB2,TSCAREB3,TSCAREB4,
  TSCAREB5,

  TOMDIE1 ,TOMDIE2 ,TOMDIE3 ,TOMDIE4 ,
  TOMDIE5 ,TOMDIE6 ,TOMDIE7 ,TOMDIE8 ,

  TOMRH1  ,TOMRH2  ,TOMRH3  ,TOMRH4  ,
  TOMRH5  ,TOMRH6  ,TOMRH7  ,TOMRH8  ,

  TOHRH1  ,TOHRH2  ,TOHRH3  ,TOHRH4  ,
  TOHRH5  ,TOHRH6  ,TOHRH7  ,TOHRH8  ,




SPEARDOWN1,SPEARDOWN2,SPEARDOWN3,SPEARDOWN4,
SPEARDOWN5,SPEARDOWN6,SPEARDOWN7,SPEARDOWN8,
SPEARDOWN9,SPEARDOWN10,SPEARDOWN11,SPEARDOWN12,
SPEARDOWN13,SPEARDOWN14,SPEARDOWN15,SPEARDOWN16,


DBLADE1,DBLADE2,DBLADE3,DBLADE4,
DBLADE5,DBLADE6,DBLADE7,DBLADE8,
DBLADE9,//DBLADE10,DBLADE11,DBLADE12,
//DBLADE13,DBLADE14,DBLADE15,DBLADE16,

FIREJETDOWN1,FIREJETDOWN2,FIREJETDOWN3,FIREJETDOWN4,
FIREJETDOWN5,FIREJETDOWN6,FIREJETDOWN7,FIREJETDOWN8,
FIREJETDOWN9,FIREJETDOWN10,FIREJETDOWN11,FIREJETDOWN12,
FIREJETDOWN13,FIREJETDOWN14,FIREJETDOWN15,FIREJETDOWN16,
FIREJETDOWN17,FIREJETDOWN18,FIREJETDOWN19,FIREJETDOWN20,
FIREJETDOWN21,FIREJETDOWN22,FIREJETDOWN23,


CRUSHUP1,CRUSHUP2,CRUSHUP3,CRUSHUP4,
CRUSHUP5,CRUSHUP6,CRUSHUP7,CRUSHUP8,



SPINUBLADE_01,SPINUBLADE_02,SPINUBLADE_03,SPINUBLADE_04,
SPINUBLADE_05,SPINUBLADE_06,SPINUBLADE_07,SPINUBLADE_08,

SPINUBLADE_09,SPINUBLADE_10,SPINUBLADE_11,SPINUBLADE_12,
SPINUBLADE_13,SPINUBLADE_14,SPINUBLADE_15,SPINUBLADE_16,




SPINDBLADE_01,SPINDBLADE_02,SPINDBLADE_03,SPINDBLADE_04,
SPINDBLADE_05,SPINDBLADE_06,SPINDBLADE_07,SPINDBLADE_08,

SPINDBLADE_09,SPINDBLADE_10,SPINDBLADE_11,SPINDBLADE_12,
SPINDBLADE_13,SPINDBLADE_14,SPINDBLADE_15,SPINDBLADE_16,



DIP11,DIP21,DIP31,
TOMLARVA1, TOMLARVA2, TOMLARVA3, TOMLARVA4,

SCOTHEAD1,SCOTHEAD2,SCOTHEAD3,SCOTHEAD4,
SCOTHEAD5,SCOTHEAD6,SCOTHEAD7,

KNIFE_STATUE1,KNIFE_STATUE2,KNIFE_STATUE3,KNIFE_STATUE4,
KNIFE_STATUE5,KNIFE_STATUE6,KNIFE_STATUE7,KNIFE_STATUE8,

EMPTY_STATUE1,EMPTY_STATUE2,EMPTY_STATUE3,EMPTY_STATUE4,
EMPTY_STATUE5,EMPTY_STATUE6,EMPTY_STATUE7,EMPTY_STATUE8,


BAT1,BAT2,BAT3,BAT4,
BAT5,BAT6,BAT7,BAT8,
BAT9,BAT10,BAT11,BAT12,
BAT13,BAT14,BAT15,BAT16,

DOGPOWERUP1,   DOGPOWERUP2,   DOGPOWERUP3, DOGPOWERUP4,
DOGPOWERUP5,   DOGPOWERUP6,   DOGPOWERUP7, DOGPOWERUP8,


THREEUP01,     THREEUP02,     THREEUP03,	THREEUP04,
THREEUP05,     THREEUP06,     THREEUP07,	THREEUP08,



BOULDER11,BOULDER21,BOULDER31,BOULDER41,

BDROP1,BDROP2,BDROP3,BDROP4,BDROP5,BDROP6,BDROP7,BDROP8,BDROP9,
BDROP10,BDROP11,

BSINK1,BSINK2,BSINK3,BSINK4,BSINK5,BSINK6,BSINK7,BSINK8,BSINK9,


GUNRISE11,GUNRISE12,GUNRISE13,GUNRISE14,
GUNRISE15,GUNRISE16,GUNRISE17,GUNRISE18,

GUNRISE21,GUNRISE22,GUNRISE23,GUNRISE24,
GUNRISE25,GUNRISE26,GUNRISE27,GUNRISE28,

GUNRISE31,GUNRISE32,GUNRISE33,GUNRISE34,
GUNRISE35,GUNRISE36,GUNRISE37,GUNRISE38,

GUNRISE41,GUNRISE42,GUNRISE43,GUNRISE44,
GUNRISE45,GUNRISE46,GUNRISE47,GUNRISE48,

GUNRISE51,GUNRISE52,GUNRISE53,GUNRISE54,
GUNRISE55,GUNRISE56,GUNRISE57,GUNRISE58,

GUNFIRE1,GUNFIRE2,GUNFIRE3,GUNFIRE4,
GUNFIRE5,GUNFIRE6,GUNFIRE7,GUNFIRE8,

GUNDEAD1,GUNDEAD2,


FOURWAY01,FOURWAY02,FOURWAY03,FOURWAY04,
FOURWAY05,FOURWAY06,FOURWAY07,FOURWAY08,
FOURWAYFIRE01,FOURWAYFIRE02,FOURWAYFIRE03,FOURWAYFIRE04,
FOURWAYFIRE05,FOURWAYFIRE06,FOURWAYFIRE07,FOURWAYFIRE08,




TOMLIGHTNING1,TOMLIGHTNING2,TOMLIGHTNING3,TOMLIGHTNING4,
TOMLIGHTNING5,TOMLIGHTNING6,TOMLIGHTNING7,TOMLIGHTNING8,

TOMSPHERE1,TOMSPHERE2,TOMSPHERE3,TOMSPHERE4,

TOMHANDBALL1,TOMHANDBALL2,

TOMFACEBALL1,TOMFACEBALL2,

TOMFLOORSPARK1,TOMFLOORSPARK2,TOMFLOORSPARK3,TOMFLOORSPARK4,


TOMSPIT1,TOMSPIT2,TOMSPIT3,TOMSPIT4,

SPITHIT1,SPITHIT2,SPITHIT3,SPITHIT4,



MONKFIRE1,MONKFIRE2,MONKFIRE3,MONKFIRE4,


BATBLAST1,BATBLAST2,BATBLAST3,BATBLAST4,

KESSPHERE1,KESSPHERE2,KESSPHERE3,KESSPHERE4,
KESSPHERE5,KESSPHERE6,KESSPHERE7,KESSPHERE8,

SPR_BOLO1,SPR_BOLO2,SPR_BOLO3,SPR_BOLO4,

//MED
DOPE1,DOPE2,DOPE3,DOPE4,DOPE5,DOPE6,DOPE7,DOPE8,
//CRUSHUP9,CRUSHUP10,CRUSHUP11,CRUSHUP12,
//CRUSHUP13,CRUSHUP14,CRUSHUP15,CRUSHUP16,


/*****************  Highguard 2 (Steve H.)  ***************************************/


SPR_HIGHGRD2_SHOOT1,SPR_HIGHGRD2_SHOOT2,SPR_HIGHGRD2_SHOOT3,SPR_HIGHGRD2_SHOOT4,


SPR_HIGHGRD2_S1,SPR_HIGHGRD2_S2,SPR_HIGHGRD2_S3,SPR_HIGHGRD2_S4,
SPR_HIGHGRD2_S5,SPR_HIGHGRD2_S6,SPR_HIGHGRD2_S7,SPR_HIGHGRD2_S8,

SPR_HIGHGRD2_W11,SPR_HIGHGRD2_W12,SPR_HIGHGRD2_W13,SPR_HIGHGRD2_W14,
SPR_HIGHGRD2_W15,SPR_HIGHGRD2_W16,SPR_HIGHGRD2_W17,SPR_HIGHGRD2_W18,

SPR_HIGHGRD2_W21,SPR_HIGHGRD2_W22,SPR_HIGHGRD2_W23,SPR_HIGHGRD2_W24,
SPR_HIGHGRD2_W25,SPR_HIGHGRD2_W26,SPR_HIGHGRD2_W27,SPR_HIGHGRD2_W28,

SPR_HIGHGRD2_W31,SPR_HIGHGRD2_W32,SPR_HIGHGRD2_W33,SPR_HIGHGRD2_W34,
SPR_HIGHGRD2_W35,SPR_HIGHGRD2_W36,SPR_HIGHGRD2_W37,SPR_HIGHGRD2_W38,

SPR_HIGHGRD2_W41,SPR_HIGHGRD2_W42,SPR_HIGHGRD2_W43,SPR_HIGHGRD2_W44,
SPR_HIGHGRD2_W45,SPR_HIGHGRD2_W46,SPR_HIGHGRD2_W47,SPR_HIGHGRD2_W48,


SPR_HIGHGRD2_PAIN1,SPR_HIGHGRD2_PAIN2,
SPR_HIGHGRD2_DIE1,SPR_HIGHGRD2_DIE2,SPR_HIGHGRD2_DIE3,SPR_HIGHGRD2_DIE4,
SPR_HIGHGRD2_DIE5,SPR_HIGHGRD2_DEAD,

SPR_HIGHGRD2_WPAIN1,SPR_HIGHGRD2_WPAIN2,
SPR_HIGHGRD2_WDIE1,SPR_HIGHGRD2_WDIE2,SPR_HIGHGRD2_WDIE3,SPR_HIGHGRD2_WDIE4,
SPR_HIGHGRD2_WDIE5,SPR_HIGHGRD2_WDEAD,

/*
SPR_HIGHGRD2_USE11,SPR_HIGHGRD2_USE12,SPR_HIGHGRD2_USE13,SPR_HIGHGRD2_USE14,
SPR_HIGHGRD2_USE15,SPR_HIGHGRD2_USE16,SPR_HIGHGRD2_USE17,SPR_HIGHGRD2_USE18,

SPR_HIGHGRD2_USE21,SPR_HIGHGRD2_USE22,SPR_HIGHGRD2_USE23,SPR_HIGHGRD2_USE24,
SPR_HIGHGRD2_USE25,SPR_HIGHGRD2_USE26,SPR_HIGHGRD2_USE27,SPR_HIGHGRD2_USE28,
*/


/****************** Alt. Overpatrol (Pat) ***********************************/

SPR_PAT_SHOOT1,SPR_PAT_SHOOT2,SPR_PAT_SHOOT3,SPR_PAT_SHOOT4,

SPR_PAT_BOLOSHOOT1,SPR_PAT_BOLOSHOOT2,SPR_PAT_BOLOSHOOT3,SPR_PAT_BOLOSHOOT4,
SPR_PAT_BOLOSHOOT5,

SPR_PAT_S1,SPR_PAT_S2,SPR_PAT_S3,SPR_PAT_S4,
SPR_PAT_S5,SPR_PAT_S6,SPR_PAT_S7,SPR_PAT_S8,

SPR_PAT_W11,SPR_PAT_W12,SPR_PAT_W13,SPR_PAT_W14,
SPR_PAT_W15,SPR_PAT_W16,SPR_PAT_W17,SPR_PAT_W18,

SPR_PAT_W21,SPR_PAT_W22,SPR_PAT_W23,SPR_PAT_W24,
SPR_PAT_W25,SPR_PAT_W26,SPR_PAT_W27,SPR_PAT_W28,

SPR_PAT_W31,SPR_PAT_W32,SPR_PAT_W33,SPR_PAT_W34,
SPR_PAT_W35,SPR_PAT_W36,SPR_PAT_W37,SPR_PAT_W38,

SPR_PAT_W41,SPR_PAT_W42,SPR_PAT_W43,SPR_PAT_W44,
SPR_PAT_W45,SPR_PAT_W46,SPR_PAT_W47,SPR_PAT_W48,


SPR_PAT_PAIN1,SPR_PAT_PAIN2,
SPR_PAT_ALTDIE1,SPR_PAT_ALTDIE2,SPR_PAT_ALTDIE3,
SPR_PAT_ALTDIE4,SPR_PAT_ALTDIE5,SPR_PAT_ALTDEAD,

SPR_PAT_WPAIN1,SPR_PAT_WPAIN2,
SPR_PAT_WALTDIE1,SPR_PAT_WALTDIE2,SPR_PAT_WALTDIE3,
SPR_PAT_WALTDIE4,SPR_PAT_WALTDIE5,SPR_PAT_WALTDEAD,

SPR_PAT_DIE1,SPR_PAT_DIE2,SPR_PAT_DIE3,
SPR_PAT_DIE4,SPR_PAT_DIE5,SPR_PAT_DIE6,
SPR_PAT_DEAD,


/******************** Alt. Lightning Guard (Ann) **********************************/


SPR_ANN_SHOOT1,SPR_ANN_SHOOT2,SPR_ANN_SHOOT3,SPR_ANN_SHOOT4,
SPR_ANN_KSHOOT1,SPR_ANN_KSHOOT2,SPR_ANN_KSHOOT3,


SPR_ANN_S1,SPR_ANN_S2,SPR_ANN_S3,SPR_ANN_S4,
SPR_ANN_S5,SPR_ANN_S6,SPR_ANN_S7,SPR_ANN_S8,

SPR_ANN_W11,SPR_ANN_W12,SPR_ANN_W13,SPR_ANN_W14,
SPR_ANN_W15,SPR_ANN_W16,SPR_ANN_W17,SPR_ANN_W18,

SPR_ANN_W21,SPR_ANN_W22,SPR_ANN_W23,SPR_ANN_W24,
SPR_ANN_W25,SPR_ANN_W26,SPR_ANN_W27,SPR_ANN_W28,

SPR_ANN_W31,SPR_ANN_W32,SPR_ANN_W33,SPR_ANN_W34,
SPR_ANN_W35,SPR_ANN_W36,SPR_ANN_W37,SPR_ANN_W38,

SPR_ANN_W41,SPR_ANN_W42,SPR_ANN_W43,SPR_ANN_W44,
SPR_ANN_W45,SPR_ANN_W46,SPR_ANN_W47,SPR_ANN_W48,


SPR_ANN_PAIN1,SPR_ANN_PAIN2,
SPR_ANN_DIE1,SPR_ANN_DIE2,SPR_ANN_DIE3,
SPR_ANN_DIE4,SPR_ANN_DEAD1,SPR_ANN_DEAD2,SPR_ANN_DEAD3,

SPR_ANN_WPAIN1,SPR_ANN_WPAIN2,
SPR_ANN_WDIE1,SPR_ANN_WDIE2,SPR_ANN_WDIE3,
SPR_ANN_WDIE4,SPR_ANN_WDEAD1,SPR_ANN_WDEAD2,SPR_ANN_WDEAD3,

SPR_ANN_RROLL1,SPR_ANN_RROLL2,SPR_ANN_RROLL3,SPR_ANN_RROLL4,
SPR_ANN_RROLL5,SPR_ANN_RROLL6,

SPR_ANN_LROLL1,SPR_ANN_LROLL2,SPR_ANN_LROLL3,SPR_ANN_LROLL4,
SPR_ANN_LROLL5,SPR_ANN_LROLL6,


/********************** Alt. Blitzguard (William) **********************************/


SPR_WILLIAM_SHOOT1,SPR_WILLIAM_SHOOT2,SPR_WILLIAM_SHOOT3,SPR_WILLIAM_SHOOT4,

SPR_WILLIAM_S1,SPR_WILLIAM_S2,SPR_WILLIAM_S3,SPR_WILLIAM_S4,
SPR_WILLIAM_S5,SPR_WILLIAM_S6,SPR_WILLIAM_S7,SPR_WILLIAM_S8,

SPR_WILLIAM_W11,SPR_WILLIAM_W12,SPR_WILLIAM_W13,SPR_WILLIAM_W14,
SPR_WILLIAM_W15,SPR_WILLIAM_W16,SPR_WILLIAM_W17,SPR_WILLIAM_W18,

SPR_WILLIAM_W21,SPR_WILLIAM_W22,SPR_WILLIAM_W23,SPR_WILLIAM_W24,
SPR_WILLIAM_W25,SPR_WILLIAM_W26,SPR_WILLIAM_W27,SPR_WILLIAM_W28,

SPR_WILLIAM_W31,SPR_WILLIAM_W32,SPR_WILLIAM_W33,SPR_WILLIAM_W34,
SPR_WILLIAM_W35,SPR_WILLIAM_W36,SPR_WILLIAM_W37,SPR_WILLIAM_W38,

SPR_WILLIAM_W41,SPR_WILLIAM_W42,SPR_WILLIAM_W43,SPR_WILLIAM_W44,
SPR_WILLIAM_W45,SPR_WILLIAM_W46,SPR_WILLIAM_W47,SPR_WILLIAM_W48,


SPR_WILLIAM_PAIN1, SPR_WILLIAM_PAIN2,SPR_WILLIAM_DIE1,SPR_WILLIAM_DIE2,
SPR_WILLIAM_DIE3,SPR_WILLIAM_DIE4,SPR_WILLIAM_DEAD1, SPR_WILLIAM_DEAD2,

SPR_WILLIAM_WPAIN1, SPR_WILLIAM_WPAIN2,SPR_WILLIAM_WDIE1,SPR_WILLIAM_WDIE2,
SPR_WILLIAM_WDIE3,SPR_WILLIAM_WDIE4,SPR_WILLIAM_WDEAD1, SPR_WILLIAM_WDEAD2,

SPR_WILLIAM_RISE1, SPR_WILLIAM_RISE2, SPR_WILLIAM_RISE3, SPR_WILLIAM_RISE4,

SPR_WILLIAM_USE11,SPR_WILLIAM_USE12,SPR_WILLIAM_USE13,SPR_WILLIAM_USE14,
SPR_WILLIAM_USE15,SPR_WILLIAM_USE16,SPR_WILLIAM_USE17,SPR_WILLIAM_USE18,

SPR_WILLIAM_USE21,SPR_WILLIAM_USE22,SPR_WILLIAM_USE23,SPR_WILLIAM_USE24,
SPR_WILLIAM_USE25,SPR_WILLIAM_USE26,SPR_WILLIAM_USE27,SPR_WILLIAM_USE28,

/******************* Fire monk (Mark)**********************************/


SPR_MARK_CAST1,SPR_MARK_CAST2,SPR_MARK_CAST3,SPR_MARK_CAST4,
SPR_MARK_CAST5,SPR_MARK_CAST6,SPR_MARK_CAST7,

SPR_MARK_S1,SPR_MARK_S2,SPR_MARK_S3,SPR_MARK_S4,
SPR_MARK_S5,SPR_MARK_S6,SPR_MARK_S7,SPR_MARK_S8,

SPR_MARK_W11,SPR_MARK_W12,SPR_MARK_W13,SPR_MARK_W14,
SPR_MARK_W15,SPR_MARK_W16,SPR_MARK_W17,SPR_MARK_W18,

SPR_MARK_W21,SPR_MARK_W22,SPR_MARK_W23,SPR_MARK_W24,
SPR_MARK_W25,SPR_MARK_W26,SPR_MARK_W27,SPR_MARK_W28,

SPR_MARK_W31,SPR_MARK_W32,SPR_MARK_W33,SPR_MARK_W34,
SPR_MARK_W35,SPR_MARK_W36,SPR_MARK_W37,SPR_MARK_W38,

SPR_MARK_W41,SPR_MARK_W42,SPR_MARK_W43,SPR_MARK_W44,
SPR_MARK_W45,SPR_MARK_W46,SPR_MARK_W47,SPR_MARK_W48,

SPR_MARK_PAIN1,SPR_MARK_PAIN2,

SPR_MARK_DIE1,SPR_MARK_DIE2,SPR_MARK_DIE3,SPR_MARK_DIE4,
SPR_MARK_DEAD1,SPR_MARK_DEAD2,SPR_MARK_DEAD3,SPR_MARK_DEAD4,
SPR_MARK_DEAD5,SPR_MARK_DEAD6,SPR_MARK_DEAD7,


/*
SPR_FIREMONK_USE11,SPR_FIREMONK_USE12,SPR_FIREMONK_USE13,SPR_FIREMONK_USE14,
SPR_FIREMONK_USE15,SPR_FIREMONK_USE16,SPR_FIREMONK_USE17,SPR_FIREMONK_USE18,

SPR_FIREMONK_USE21,SPR_FIREMONK_USE22,SPR_FIREMONK_USE23,SPR_FIREMONK_USE24,
SPR_FIREMONK_USE25,SPR_FIREMONK_USE26,SPR_FIREMONK_USE27,SPR_FIREMONK_USE28,
*/

//#endif

} actornames_t;

typedef enum
 {GUNSTART_LABEL,

#if (SHAREWARE == 0)

  W_KNIFE,WK2,WK3,WK4,WK5,WK6,WK7,WK8,WK9,WK10,

  W_BMALEPISTOL1,xab,xcd,
  W_BMRIGHTPISTOL1,xef,xgh,
  W_BMLEFTPISTOL1,xij,xkl,
#endif

  W_MALEPISTOL1,x5,x6,
  W_MRIGHTPISTOL1,x7,x8,
  W_MLEFTPISTOL1,x9,x10,

#if (SHAREWARE == 0)
  W_FEMALEPISTOL1,x11,x12,
  W_FRIGHTPISTOL1,x13,x14,
  W_FLEFTPISTOL1,x15,x16,
#endif

  W_MP40,FN7,FN8,
  W_BAZOOKA,FN22,fn104,fn105,
  W_HEATSEEKER,FN80,fn108,fn109,
  W_DRUNK,FN72,FN73,FN74,
  W_FIREBOMB,FN33,fn106,fn107,
  W_FIREWALL,FN66,FN67,
  W_GODHAND,ded1,ded2,ded3,ded4,ded5,ded6,ded7,

#if (SHAREWARE == 0)

  W_SPLIT,FN69,FN70,FN71,
  W_KES,fn101,fn102,fn103,x99,x100,
  W_BAT,fn111,fn112,fn113,fn114,fn115,fn117,
  W_DOG,bite1,bite2,bite3,nose1,nose2,nose3,nose4,paw1,paw2,paw3,paw4
#endif
 } weaponsprites;


#define SPR_DDOG_FIRE1 SPR_DDOG_JUMP1
#define SPR_DDOG_FIRE2 SPR_DDOG_JUMP2
#define SPR_DDOG_BOLTS2 SPR_DDOG_BOLTS1
#define SPR_DDOG_BOLTS3 SPR_DDOG_BOLTS1
#define SPR_DDOG_BOLTS4 SPR_DDOG_BOLTS1
#define SPR_DDOG_HIT1 -1
#define SPR_DDOG_HIT2 -1
#define SPR_DDOG_HIT3 -1
#define SPR_DDOG_PAIN1 SPR_DDOG_DIE1
#define SPR_DDOG_PAIN2 SPR_DDOG_DIE1


#define SPR_BLITZ_USE SPR_BLITZ_S1

#define SPR_BLITZ_STEAL1 SPR_BLITZ_USE11
#define SPR_BLITZ_STEAL2 SPR_BLITZ_USE12



#define SPR_GRENADE_HIT1 -1
#define SPR_GRENADE_HIT2 -1
#define SPR_GRENADE_HIT3 -1

#define SPR_FIREBALL_HIT1 -1
#define SPR_FIREBALL_HIT2 -1
#define SPR_FIREBALL_HIT3 -1

#define SPR_ROBOGRD_SHURIKEN1 SPR_BSTAR1
#define SPR_ROBOGRD_SHURIKEN2 SPR_BSTAR2
#define SPR_ROBOGRD_SHURIKEN3 SPR_BSTAR3
#define SPR_ROBOGRD_SHURIKEN4 SPR_BSTAR4
#define SPR_SHURIKEN_HIT1 -1
#define SPR_SHURIKEN_HIT2 -1
#define SPR_SHURIKEN_HIT3 -1


#define SPR_ESAU_USE SPR_ESAU_S1
#define SPR_ESAU_WINS SPR_ESAU_S1
#define SPR_ESAU_DIE4 SPR_ESAU_DIE3
#define SPR_ESAU_DIE5 SPR_ESAU_DIE3
#define SPR_ESAU_DIE6 SPR_ESAU_DIE3
#define SPR_ESAU_DIE7 SPR_ESAU_DIE3


#define SPR_MISSILEHIT1 -1
#define SPR_MISSILEHIT2 -1
#define SPR_MISSILEHIT3 -1
#define SPR_HEINRICH_DIE4 SPR_HEINRICH_DIE3
#define SPR_HEINRICH_DIE5 SPR_HEINRICH_DIE3
#define SPR_HEINRICH_DIE6 SPR_HEINRICH_DIE3
#define SPR_HEINRICH_DIE7 SPR_HEINRICH_DIE3
#define SPR_HEINRICH_DEAD SPR_HEINRICH_DIE3
#define SPR_HEINRICH_MINE SPR_HEINRICH_S1
#define SPR_HEINRICH_USE SPR_HEINRICH_S1

#define SPR_DMFBALL11 SPR_FIREBALL1
#define SPR_DMFBALL21 SPR_FIREBALL2
#define SPR_DMFBALL31 SPR_FIREBALL3
#define SPR_DMFBALL41 SPR_FIREBALL4
#define SPR_DMFBALLHIT1 -1
#define SPR_DMFBALLHIT2 -1
#define SPR_DMFBALLHIT3 -1
#define SPR_DARKMONK_DIE4 SPR_DARKMONK_DIE3
#define SPR_DARKMONK_DIE5 SPR_DARKMONK_DIE3
#define SPR_DARKMONK_DIE6 SPR_DARKMONK_DIE3
#define SPR_DARKMONK_DIE7 SPR_DARKMONK_DIE3
#define SPR_DARKMONK_PAIN1 SPR_DARKMONK_DIE1
#define SPR_DARKMONK_PAIN2 SPR_DARKMONK_DIE5

#define SPR_OSHURHIT1 -1
#define SPR_OSHURHIT2 -1
#define SPR_OSHURHIT3 -1

#define SPR_OSHUR1 SPR_BSTAR1
#define SPR_OSHUR2 SPR_BSTAR2
#define SPR_OSHUR3 SPR_BSTAR4
#define SPR_OSHUR4 SPR_BSTAR4

#define SPR_WALLSHOOT SPR_WALLSTAND1
#define SPR_WALLPATH1 SPR_WALLSTAND1

#endif
