/*
Copyright (C) 1994-1995 Apogee Software, Ltd.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#ifndef _cin_evnt_public
#define _cin_evnt_public

#include "cin_glob.h"
#include "cin_def.h"

extern eventtype * firstevent;
extern eventtype * lastevent;

void AddEvent (eventtype * event);
void DeleteEvent(eventtype * event);

eventtype * GetNewEvent ( void );
void StartupEvents ( void );
void ShutdownEvents ( void );
eventtype * CreateEvent ( int time, int type );
void ParseEvent ( int time );
void UpdateCinematicEvents ( int time );
void PrecacheCinematic ( void );

#endif

