/*
Copyright (C) 1994-1995 Apogee Software, Ltd.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#ifndef _keyb
#define _keyb

#define	sc_None			0
#define	sc_Bad			0xff
#define	sc_Comma    	0x33
#define	sc_Period		0x34
#define	sc_Return		0x1c
#define	sc_Enter		   sc_Return
#define	sc_Escape		0x01
#define	sc_Space		   0x39
#define	sc_BackSpace	0x0e
#define	sc_Tab			0x0f
#define	sc_Alt			0x38
#define	sc_Control		0x1d
#define	sc_CapsLock		0x3a
#define	sc_LShift		0x2a
#define	sc_RShift		0x36
#define	sc_UpArrow		0x48
#define	sc_DownArrow	0x50
#define	sc_LeftArrow	0x4b
#define	sc_RightArrow	0x4d
#define	sc_Insert		0x52
#define	sc_Delete		0x53
#define	sc_Home			0x47
#define	sc_End			0x4f
#define	sc_PgUp			0x49
#define	sc_PgDn			0x51
#define	sc_F1				0x3b
#define	sc_F2				0x3c
#define	sc_F3				0x3d
#define	sc_F4				0x3e
#define	sc_F5				0x3f
#define	sc_F6				0x40
#define	sc_F7				0x41
#define	sc_F8				0x42
#define	sc_F9				0x43
#define	sc_F10			0x44
#define	sc_F11			0x57
#define	sc_F12			0x58
#define  sc_PrintScreen 0x37

#define  sc_OpenBracket      0x1a
#define  sc_CloseBracket     0x1b

#define  sc_1           0x02
#define	sc_2				0x03
#define	sc_3				0x04
#define	sc_4				0x05
#define	sc_5				0x06
#define	sc_6				0x07
#define	sc_7				0x08
#define	sc_8				0x09
#define	sc_9				0x0a
#define	sc_0				0x0b
#define	sc_Minus       0x0c
#define	sc_Equals      0x0d
#define	sc_Plus        0x0d

#define	sc_A				0x1e
#define	sc_B				0x30
#define	sc_C				0x2e
#define	sc_D				0x20
#define	sc_E				0x12
#define	sc_F				0x21
#define	sc_G				0x22
#define	sc_H				0x23
#define	sc_I				0x17
#define	sc_J				0x24
#define	sc_K				0x25
#define	sc_L				0x26
#define	sc_M				0x32
#define	sc_N				0x31
#define	sc_O				0x18
#define	sc_P				0x19
#define	sc_Q				0x10
#define	sc_R				0x13
#define	sc_S				0x1f
#define	sc_T				0x14
#define	sc_U				0x16
#define	sc_V				0x2f
#define	sc_W				0x11
#define	sc_X				0x2d
#define	sc_Y				0x15
#define	sc_Z				0x2c

#endif
